@@ -1037,6 +1037,7 @@ class Damos:
 
 class DamonCtx:
     ops = None
+    addr_unit = None
     targets = None
     intervals = None
     nr_regions = None
@@ -1044,8 +1045,11 @@ class DamonCtx:
     kdamond = None
 
     def __init__(self, ops='paddr', targets=None, intervals=None,
-                 nr_regions=None, schemes=None):
+                 nr_regions=None, schemes=None, addr_unit=None):
         self.ops = ops
+        if addr_unit is not None:
+            self.addr_unit = _damo_fmt_str.text_to_bytes(addr_unit)
+        self.addr_unit = addr_unit
         self.targets = targets if targets is not None else []
         for target in self.targets:
             target.context = self
@@ -1058,7 +1062,11 @@ class DamonCtx:
             scheme.context = self
 
     def to_str(self, raw, params_only=False):
-        lines = ['ops: %s' % self.ops]
+        ops_line = 'ops: %s' % self.ops
+        if self.addr_unit is not None:
+            ops_line = '%s (addr_unit %s)' % (
+                    ops_line, _damo_fmt_str.format_sz(self.addr_unit, raw))
+        lines = [ops_line]
         for idx, target in enumerate(self.targets):
             lines.append('target %d' % idx)
             lines.append(_damo_fmt_str.indent_lines(target.to_str(raw), 4))
@@ -1095,12 +1103,15 @@ class DamonCtx:
                 DamonNrRegionsRange.from_kvpairs(kv['nr_regions'])
                     if 'nr_regions' in kv else DAmonNrRegionsRange(),
                 [Damos.from_kvpairs(s) for s in kv['schemes']]
-                    if 'schemes' in kv else [])
+                    if 'schemes' in kv else [],
+                    kv['addr_unit']
+                    if 'addr_unit' in kv else None)
         return ctx
 
     def to_kvpairs(self, raw=False, omit_defaults=False, params_only=False):
         kv = collections.OrderedDict({})
         kv['ops'] = self.ops
+        kv['addr_unit'] = self.addr_unit
         kv['targets'] = [t.to_kvpairs(raw) for t in self.targets]
         if not omit_defaults or self.intervals != DamonIntervals():
             kv['intervals'] = self.intervals.to_kvpairs(raw)
@@ -1241,15 +1252,17 @@ features = ['record',       # was in DAMON patchset, but not merged in mainline
             'sz_ops_filter_passed',     # merged in v6.14-rc1
             'allow_filter',             # merged in v6.14-rc1
             'schemes_filters_hugepage_size',
-                                        # merged in mm-unstable
-            'schemes_filters_unmapped', # merged in mm-unstable
-            'intervals_goal',           # merged in mm-unstable
+                                        # merged in v6.15-rc1
+            'schemes_filters_unmapped', # merged in v6.15-rc1
+            'intervals_goal',           # merged in v6.15-rc1
             'schemes_filters_core_ops_dirs',
-                                        # merged in mm-unstable
+                                        # merged in v6.15-rc1
+            'schemes_filters_active',
+                                        # merged in v6.15-rc1
+
             'schemes_quota_goal_node_mem_used_free',
                                         # under development
-            'schemes_filters_active',
-                                        # under development
+            'addr_unit'                 # under development
             ]
 
 _damon_fs = None
