@@ -471,7 +471,8 @@ def damon_ctx_for(args, idx):
     ops = args.ops[idx]
 
     try:
-        ctx = _damon.DamonCtx(ops, None, intervals, nr_regions, schemes=[])
+        ctx = _damon.DamonCtx(ops, None, intervals, nr_regions, schemes=[],
+                              addr_unit=args.ops_addr_unit)
         return ctx, None
     except Exception as e:
         return None, 'Creating context from arguments failed (%s)' % e
@@ -785,7 +786,8 @@ def set_common_argparser(parser):
     parser.add_argument('--damon_interface_DEPRECATED',
             choices=['sysfs', 'debugfs', 'auto'],
             default='auto',
-            help='underlying DAMON interface to use (!! DEPRECATED)')
+            # underlying DAMON interface to use (!! DEPRECATED)
+            help=argparse.SUPPRESS)
     parser.add_argument('--debug_damon', action='store_true',
             help='Print debugging log')
 
@@ -834,6 +836,9 @@ def set_monitoring_argparser(parser, hide_help=False):
                         action='append',
                         help='monitoring operations set'
                         if not hide_help else argparse.SUPPRESS)
+    parser.add_argument('--ops_addr_unit', metavar='<bytes>',
+                        help='operations set address unit'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--target_pid', type=int, metavar='<pid>',
                         action='append',
                         help='pid of monitoring target process'
@@ -849,10 +854,10 @@ def set_monitoring_argparser(parser, hide_help=False):
             if not hide_help else argparse.SUPPRESS)
     set_monitoring_attrs_argparser(parser, hide_help)
     parser.add_argument(
-            '--nr_targets', nargs='+', type=int,
+            '--nr_targets', metavar='<number>', nargs='+', type=int,
             help='number of monitoring targets for each context (in order)'
             if not hide_help else argparse.SUPPRESS)
-    parser.add_argument('--nr_ctxs', nargs='+', type=int,
+    parser.add_argument('--nr_ctxs', metavar='<number>', nargs='+', type=int,
                         help='number of contexts for each kdamond (in order)'
                         if not hide_help else argparse.SUPPRESS)
 
@@ -938,7 +943,7 @@ def set_damos_argparser(parser, hide_help):
                 '<low mark (permil)>'),
             help='damos watermarks'
             if not hide_help else argparse.SUPPRESS)
-    parser.add_argument('--nr_schemes', nargs='+', type=int,
+    parser.add_argument('--nr_schemes', metavar='<number>', nargs='+', type=int,
                         help='number of schemes for each context (in order)'
                         if not hide_help else argparse.SUPPRESS)
 
