@@ -495,6 +495,12 @@ def write_context_dir(dir_path, context):
     if err is not None:
         return err
 
+    if context.addr_unit is not None:
+        err = _damo_fs.write_file(os.path.join(dir_path, 'addr_unit'),
+                                  context.addr_unit)
+        if err is not None:
+            return err
+
     err = write_monitoring_attrs_dir(
             os.path.join(dir_path, 'monitoring_attrs'), context)
     if err is not None:
@@ -730,6 +736,10 @@ def files_content_to_context(files_content):
             int(nr_regions_content['min']),
             int(nr_regions_content['max']))
     ops = files_content['operations'].strip()
+    if 'addr_unit' in files_content:
+        addr_unit = files_content['addr_unit']
+    else:
+        addr_unit = None
 
     targets_content = files_content['targets']
     targets = [files_content_to_target(content)
@@ -741,7 +751,8 @@ def files_content_to_context(files_content):
             for content in numbered_dirs_content(
                 schemes_content, 'nr_schemes')]
 
-    return _damon.DamonCtx(ops, targets, intervals, nr_regions, schemes)
+    return _damon.DamonCtx(ops, targets, intervals, nr_regions, schemes,
+                           addr_unit=addr_unit)
 
 def files_content_to_kdamond(files_content):
     contexts_content = files_content['contexts']
@@ -859,11 +870,13 @@ def infer_damon_version():
                                 )])])]
     err = stage_kdamonds(kdamonds)
     if err is None:
-        if os.path.isdir(
+        if os.path.isfile(os.path.join(ctx_dir_of(0, 0), 'addr_unit')):
+            version = '>v6.15'
+        elif os.path.isdir(
                 os.path.join(ctx_dir_of(0, 0),
                              'monitoring_attrs', 'intervals', 'intervals_goal')):
             version = '>v6.14'
-        if os.path.isfile(os.path.join(scheme_dir_of(0, 0, 0), 'stats',
+        elif os.path.isfile(os.path.join(scheme_dir_of(0, 0, 0), 'stats',
                                        'sz_ops_filter_passed')):
             version = '>v6.13'
         elif os.path.isfile(os.path.join(scheme_dir_of(0, 0, 0), 'target_nid')):
@@ -989,7 +1002,10 @@ def update_supported_features():
             os.path.join(scheme_dir_of(0, 0, 0), 'core_filters')):
         feature_supports['schemes_filters_core_ops_dirs'] = True
 
-    # todo: check unmapped filter support by trying writing it to filter type
+        # unmapped and active pages DAMOS filters are merged into v6.15
+        # together with core_ops_dirs
+        feature_supports['schemes_filters_unmapped'] = True
+        feature_supports['schemes_filters_active'] = True
 
     if feature_supports['schemes_quota_goals'] is True:
         kdamonds_for_feature_check = [
@@ -1011,6 +1027,9 @@ def update_supported_features():
                              'nid')):
             feature_supports['schemes_quota_goal_node_mem_used_free'] = True
 
+    if os.path.isfile(os.path.join(ctx_dir_of(0, 0), 'addr_unit')):
+        feature_supports['addr_unit'] = True
+
     avail_ops, err = _avail_ops()
     if err == None:
         for ops in ['vaddr', 'paddr', 'fvaddr']:
