@@ -1,11 +1,14 @@
 # SPDX-License-Identifier: GPL-2.0
 
 import argparse
+import code
 import copy
 import json
 import math
 import os
 import signal
+import sys
+import tempfile
 import time
 
 import _damo_ascii_color
@@ -124,7 +127,6 @@ snapshot_formatters = [
                   recency_hist_str(snapshot, record, fmt, True),
                   ' '.join(['last accessed time to total size of',
                             'DAMOS filters (df) passed regions histogram'])),
-
         Formatter('<heatmap>',
                   lambda snapshot, record, fmt:
                   heatmap_str(snapshot, record, fmt),
@@ -302,30 +304,38 @@ def filters_pass_type_of(record):
         return 'n/a'
     text = ', '.join(['%s' % f for f in ops_filters])
     if ops_filters[-1].allow is True:
-        text += '\n# WARN: Allow filters at the end of the list means nothing'
+        text += '\n# WARN: Allow filters at the end of the list means nothing on <6.15'
     return text
 
-def get_linearscale_hist_ranges(minv, maxv, nr_ranges):
+def get_linearscale_hist_ranges(minv, maxv, nr_ranges, cumulate):
     hist_ranges = []
     total_interval = maxv + 1 - minv
     interval = max(math.ceil(total_interval / nr_ranges), 1)
     for i in range(nr_ranges):
-        hist_ranges.append([minv + interval * i, minv + interval * (i + 1)])
+        if cumulate:
+            start_range = minv
+        else:
+            start_range = minv + interval * i
+        hist_ranges.append([start_range, minv + interval * (i + 1)])
     return hist_ranges
 
-def get_logscale_hist_ranges(minv, maxv, nr_ranges):
+def get_logscale_hist_ranges(minv, maxv, nr_ranges, cumulate):
     initial_interval = max(int((maxv - minv) / 2**nr_ranges), 1)
     # start from 1 second interval
     ranges = [[minv, minv + initial_interval]]
     while ranges[-1][-1] < maxv:
         last_interval = ranges[-1][1] - ranges[-1][0]
-        ranges.append([ranges[-1][-1], ranges[-1][-1] + last_interval * 2])
+        if cumulate:
+            start_range = minv
+        else:
+            start_range = ranges[-1][-1]
+        ranges.append([start_range, ranges[-1][-1] + last_interval * 2])
     return ranges
 
-def get_hist_ranges(minv, maxv, nr_ranges, logscale):
+def get_hist_ranges(minv, maxv, nr_ranges, logscale, cumulate):
     if logscale:
-        return get_logscale_hist_ranges(minv, maxv, nr_ranges)
-    return get_linearscale_hist_ranges(minv, maxv, nr_ranges)
+        return get_logscale_hist_ranges(minv, maxv, nr_ranges, cumulate)
+    return get_linearscale_hist_ranges(minv, maxv, nr_ranges, cumulate)
 
 def histogram_str(hist):
     """
@@ -335,50 +345,75 @@ def histogram_str(hist):
     Returns:
         str: A string of the histogram that formatted in a user-friendly way.
     """
-    sizes = [entry[2] for entry in hist]
-    min_sz = min(sizes)
-    max_sz = max(sizes)
-    max_trange_str = max([len(entry[0]) for entry in hist])
-    max_sz_str = max([len(entry[1]) for entry in hist])
+    yvals = [entry[2] for entry in hist]
+    min_yval = min(yvals)
+    max_yval = max(yvals)
+    max_xval_range_str = max([len(entry[0]) for entry in hist])
+    max_yval_str = max([len(entry[1]) for entry in hist])
     max_dots = 20
-    sz_interval = max(int((max_sz - min_sz) / max_dots), 1)
+    yval_dot_interval = max(int((max_yval - min_yval) / max_dots), 1)
     lines = []
-    for trange_str, sz_str, sz in hist:
-        trange_str = '%s%s' % (trange_str,
-                               ' ' * (max_trange_str - len(trange_str)))
-        sz_str = '%s%s' % (sz_str,
-                           ' ' * (max_sz_str - len(sz_str)))
+    for xrange_str, y_str, yval in hist:
+        xrange_str = '%s%s' % (xrange_str,
+                               ' ' * (max_xval_range_str - len(xrange_str)))
+        y_str = '%s%s' % (y_str,
+                           ' ' * (max_yval_str - len(y_str)))
 
-        nr_dots = min(math.ceil((sz - min_sz) / sz_interval), max_dots)
+        nr_dots = min(math.ceil((yval - min_yval) / yval_dot_interval), max_dots)
         bar = '|%s%s|' % ('*' * nr_dots, ' ' * (max_dots - nr_dots))
-        lines.append('%s %s %s' % (trange_str, sz_str, bar))
+        lines.append('%s %s %s' % (xrange_str, y_str, bar))
     return '\n'.join(lines)
 
-def sz_hist_str(snapshot, fmt, df_passed_sz, get_metric_fn, fmt_metric_fn):
-    raw = fmt.raw_number
-    if len(snapshot.regions) == 0:
-        return 'no region in snapshot'
+def get_unsorted_histogram(snapshot, fmt, get_x_fn, get_y_fn):
     hist = {}
     for region in snapshot.regions:
-        metric = get_metric_fn(region, fmt)
-        if not metric in hist:
-            hist[metric] = 0
-        if df_passed_sz:
-            hist[metric] += region.sz_filter_passed
-        else:
-            hist[metric] += region.size()
+        xval = get_x_fn(region, fmt)
+        if not xval in hist:
+            hist[xval] = 0
+        hist[xval] += get_y_fn(region, fmt)
+    return hist
 
+def get_sorted_ranged_historgram(hist, fmt, fmt_x_fn, parse_x_fn, fmt_y_fn):
     metrics = sorted(hist.keys())
-    min_metric, max_metric = metrics[0], metrics[-1]
-
     hist2 = []
-    for min_m, max_m in get_hist_ranges(
-            min_metric, max_metric, 10, fmt.hist_logscale):
-        sz = sum([hist[x] for x in metrics if x >= min_m and x < max_m])
+    if fmt.hist_ranges is not None:
+        hist_ranges = []
+        for i in range(0, len(fmt.hist_ranges), 2):
+            hist_ranges.append(
+                    [parse_x_fn(fmt.hist_ranges[i]),
+                     parse_x_fn(fmt.hist_ranges[i + 1])])
+    else:
+        min_metric, max_metric = metrics[0], metrics[-1]
+        hist_ranges = get_hist_ranges(
+                min_metric, max_metric, 10, fmt.hist_logscale,
+                fmt.hist_cumulate)
+    raw = fmt.raw_number
+    for min_m, max_m in hist_ranges:
+        yval = sum([hist[x] for x in metrics if x >= min_m and x < max_m])
         metric_range_str = '[%s, %s)' % (
-                fmt_metric_fn(min_m, raw), fmt_metric_fn(max_m, raw))
-        sz_str = _damo_fmt_str.format_sz(sz, raw)
-        hist2.append([metric_range_str, sz_str, sz])
+                fmt_x_fn(min_m, raw), fmt_x_fn(max_m, raw))
+        yval_str = fmt_y_fn(yval, raw)
+        hist2.append([metric_range_str, yval_str, yval])
+    return hist2
+
+def get_sz_region(region, fmt):
+    return region.size()
+
+def get_df_passed_sz_region(region):
+    return region.sz_filter_passed
+
+def sz_hist_str(snapshot, fmt, df_passed_sz, get_metric_fn, fmt_metric_fn,
+                parse_metric_fn):
+    if len(snapshot.regions) == 0:
+        return 'no region in snapshot'
+    if df_passed_sz is True:
+        get_y_fn = get_df_passed_sz_region
+    else:
+        get_y_fn = get_sz_region
+    hist = get_unsorted_histogram(snapshot, fmt, get_metric_fn, get_y_fn)
+    hist2 = get_sorted_ranged_historgram(
+            hist, fmt, fmt_metric_fn, parse_metric_fn, _damo_fmt_str.format_sz)
+
     return histogram_str(hist2)
 
 def temperature_sz_hist_str(snapshot, record, fmt, df_passed_sz):
@@ -388,16 +423,33 @@ def temperature_sz_hist_str(snapshot, record, fmt, df_passed_sz):
         return temperature_of(region, weights)
 
     return sz_hist_str(snapshot, fmt, df_passed_sz, get_temperature,
-                       _damo_fmt_str.format_nr)
+                       _damo_fmt_str.format_nr, _damo_fmt_str.text_to_nr)
+
+def get_last_used_time(region, fmt):
+    if region.nr_accesses.percent > 0:
+        return 0
+    return region.age.usec
 
 def recency_hist_str(snapshot, record, fmt, df_passed_sz):
-    def get_last_used_time(region, fmt):
-        if region.nr_accesses.percent > 0:
-            return 0
-        return region.age.usec
-
     return sz_hist_str(snapshot, fmt, df_passed_sz, get_last_used_time,
-                       _damo_fmt_str.format_time_us)
+                       _damo_fmt_str.format_time_us, _damo_fmt_str.text_to_us)
+
+class HistFns:
+    get = None
+    fmt = None
+    parse = None
+
+    def __init__(self, get, fmt, parse):
+        self.get = get
+        self.fmt = fmt
+        self.parse = parse
+
+hist_fns_map = {
+        'recency': HistFns(get_last_used_time, _damo_fmt_str.format_time_us,
+                           _damo_fmt_str.text_to_us),
+        'sz': HistFns(get_sz_region, _damo_fmt_str.format_sz,
+                      _damo_fmt_str.text_to_bytes),
+        }
 
 def temperature_str(region, raw, fmt):
     temperature = temperature_of(region, fmt.temperature_weights)
@@ -936,7 +988,10 @@ class ReportFormat:
     sort_regions_dsc = None
     temperature_weights = None
     dont_merge_regions = None
+
     hist_logscale = None
+    hist_cumulate = None
+    hist_ranges = None
 
     format_record_head = None
     format_record_tail = None
@@ -972,6 +1027,8 @@ class ReportFormat:
         self.temperature_weights = args.temperature_weights
         self.dont_merge_regions = args.dont_merge_regions
         self.hist_logscale = args.hist_logscale
+        self.hist_cumulate = args.hist_cumulate
+        self.hist_ranges = args.hist_ranges
         self.format_record_head = args.format_record_head
         self.format_record_tail = args.format_record_tail
         self.format_snapshot_head = args.format_snapshot_head
@@ -1005,6 +1062,8 @@ class ReportFormat:
                 'temperature_weights': self.temperature_weights,
                 'dont_merge_regions': self.dont_merge_regions,
                 'hist_logscale': self.hist_logscale,
+                'hist_cumulate': self.hist_cumulate,
+                'hist_ranges': self.hist_ranges,
                 'format_record_head': self.format_record_head,
                 'format_record_tail': self.format_record_tail,
                 'format_snapshot_head': self.format_snapshot_head,
@@ -1036,6 +1095,16 @@ class ReportFormat:
             self.hist_logscale = kvpairs['hist_logscale']
         else:
             self.hist_logscale = False
+        # hist_cumulate introduced after v2.7.6
+        if 'hist_cumulate' in kvpairs:
+            self.hist_cumulate = kvpairs['hist_cumulate']
+        else:
+            self.hist_cumulate = False
+        # hist_ranges introduced after v2.7.5
+        if 'hist_ranges' in kvpairs:
+            self.hist_ranges = kvpairs['hist_ranges']
+        else:
+            self.hist_ranges = None
         self.format_record_head = kvpairs['format_record_head']
         self.format_record_tail = kvpairs['format_record_tail']
         self.format_snapshot_head = kvpairs['format_snapshot_head']
@@ -1189,13 +1258,13 @@ def set_formats(args, records):
 
 def handle_ls_keywords(args):
     if args.ls_record_format_keywords:
-        print('\n\n'.join(['%s' % f for f in record_formatters]))
+        print("use 'damo help access_format_options record_format_keywords' instead")
         return True
     if args.ls_snapshot_format_keywords:
-        print('\n\n'.join(['%s' % f for f in snapshot_formatters]))
+        print("use 'damo help access_format_options snapshot_format_keywords' instead")
         return True
     if args.ls_region_format_keywords:
-        print('\n\n'.join(['%s' % f for f in region_formatters]))
+        print("use 'damo help access_format_options region_format_keywords' instead")
         return True
     return False
 
@@ -1254,6 +1323,37 @@ def sighandler(signum, frame):
     print('\nsignal %s received' % signum)
     signal_received = True
 
+def recursive_del(dirpath):
+    for filename in os.listdir(dirpath):
+        filepath = os.path.join(dirpath, filename)
+        if os.path.isdir(filepath):
+            recursive_del(filepath)
+        else:
+            os.remove(filepath)
+    os.rmdir(dirpath)
+
+def handle_exec(cmd, records):
+    if cmd == 'interpreter':
+        code.interact(
+                local=locals(),
+                banner="DAMON records are available as 'records'",
+                exitmsg='Exit the session.')
+        return None
+    cmd_fields = cmd.split()
+    if not os.path.isfile(cmd_fields[0]):
+        return 'first token is not file'
+    script_dir_path = tempfile.mkdtemp(prefix='damo_script_')
+    script_module_name = 'damo_report_access_script'
+    script_file_name = '%s.py' % script_module_name
+    with open(cmd_fields[0], 'r') as f:
+        with open(os.path.join(script_dir_path, script_file_name), 'w') as f2:
+            f2.write(f.read())
+    sys.path.append(script_dir_path)
+    import damo_report_access_script
+    damo_report_access_script.main(records, cmd_fields)
+    recursive_del(script_dir_path)
+    return None
+
 def read_and_show(args):
     record_filter, err = _damo_records.args_to_filter(args)
     if err != None:
@@ -1304,6 +1404,15 @@ def read_and_show(args):
         if signal_received is True:
             break
 
+        if args.exec:
+            err = handle_exec(args.exec, records)
+            if err is not None:
+                print('format_script handling fail (%s)' % err)
+                exit(1)
+            time.sleep(repeat_delay)
+            read_show_count += 1
+            continue
+
         if len([r for r in records if r.intervals is None]) != 0:
             if not args.json and not args.raw_form:
                 print('some records lack the intervals information')
@@ -1356,22 +1465,28 @@ def add_fmt_args(parser, hide_help=False):
             '--sort_regions_by', nargs='+',
             choices=['address', 'access_rate', 'age', 'size', 'temperature'],
             default=['address'],
-            help='fields to sort regions by')
+            help='fields to sort regions by'
+            if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--sort_regions_dsc',
             choices=['address', 'access_rate', 'age', 'size', 'temperature',
                      'all'],
             nargs='+',
-            help='sort regions in descending order for the given keys')
+            help='sort regions in descending order for the given keys'
+            if not hide_help else argparse.SUPPRESS)
     parser.add_argument(
             '--temperature_weights', type=int, metavar='<int>', nargs=3,
             default=[0, 100, 100],
             help=' '.join(
                 ['temperature weights for size, access rate, and age',
-                 'of each region']))
-    parser.add_argument('--dont_merge_regions', action='store_true',
-            help='don\'t merge contiguous regions of same access pattern')
+                 'of each region']) if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--hist_logscale', action='store_true',
-                        help='draw histograms in logscale')
+                        help='draw histograms in logscale'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument('--hist_ranges', nargs='+',
+                        help='histogram range values'
+                        if not hide_help else argparse.SUPPRESS)
+    parser.add_argument('--hist_cumulate', action='store_true',
+                        help='draw histogram in cumulative way')
 
     # don't set default for record head and snapshot head because it depends on
     # given number of record and snapshots.  Decide those in set_formats().
@@ -1392,20 +1507,31 @@ def add_fmt_args(parser, hide_help=False):
             default=default_snapshot_tail_format,
             help='output format to show at the end of each snapshot'
             if not hide_help else argparse.SUPPRESS)
-    parser.add_argument(
-            '--snapshot_heatmap_width', default=80, type=int,
-            help='width of snapshot heatmap')
-    parser.add_argument(
-            '--snapshot_heatmap_colorset', default='gray',
-            choices=_damo_ascii_color.colorsets.keys(),
-            help='snapshot heatmap colorset')
-    parser.add_argument(
-            '--snapshot_heatmap_static', action='store_true',
-            help='draw snapshot heatmap as static')
     parser.add_argument('--format_region', metavar='<template>',
                         default=default_region_format,
                         help='output format to show for each memory region'
                         if not hide_help else argparse.SUPPRESS)
+
+    # for snapshot heatmap
+    parser.add_argument(
+            '--snapshot_heatmap_width', metavar='<number>', default=80,
+            type=int,
+            help='width of snapshot heatmap'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--snapshot_heatmap_colorset', default='gray',
+            choices=_damo_ascii_color.colorsets.keys(),
+            help='snapshot heatmap colorset'
+            if not hide_help else argparse.SUPPRESS)
+    parser.add_argument(
+            '--snapshot_heatmap_static', action='store_true',
+            help='draw snapshot heatmap as static'
+            if not hide_help else argparse.SUPPRESS)
+
+    # for region box
+    parser.add_argument('--region_box', action='store_true',
+            help='show region access pattern as a box'
+            if not hide_help else argparse.SUPPRESS)
     parser.add_argument(
             '--region_box_values',
             choices=['size', 'access_rate', 'age', 'none'], nargs=3,
@@ -1436,6 +1562,8 @@ def add_fmt_args(parser, hide_help=False):
             '--region_box_align', choices=['left', 'right'], default='left',
             help='where to align the region box'
             if not hide_help else argparse.SUPPRESS)
+
+    # misc control
     parser.add_argument(
             '--min_chars_for', nargs=2,
             metavar=('<keyword>', '<number>'), action='append',
@@ -1447,34 +1575,36 @@ def add_fmt_args(parser, hide_help=False):
                      ],
             help='minimum character for each keyword of the format'
             if not hide_help else argparse.SUPPRESS)
-    parser.add_argument('--region_box', action='store_true',
-            help='show region access pattern as a box')
     parser.add_argument('--total_sz_only', action='store_true',
             help='print only total size of the regions for each snapshot')
+    parser.add_argument('--dont_merge_regions', action='store_true',
+            help='don\'t merge contiguous regions of same access pattern'
+            if not hide_help else argparse.SUPPRESS)
+
     parser.add_argument('--raw_number', action='store_true',
             help='use machine-friendly raw numbers')
     parser.add_argument('--json', action='store_true',
             help='print in json format')
     parser.add_argument('--raw_form', action='store_true',
-                        help='print in raw format')
+                        help='print in raw format'
+                        if not hide_help else argparse.SUPPRESS)
     parser.add_argument('--ls_record_format_keywords', action='store_true',
-                        help='list available record format keywords'
-                        if not hide_help else argparse.SUPPRESS)
+                        # help='list available record format keywords'
+                        help=argparse.SUPPRESS)
     parser.add_argument('--ls_snapshot_format_keywords', action='store_true',
-                        help='list available snapshot format keywords'
-                        if not hide_help else argparse.SUPPRESS)
+                        # help='list available snapshot format keywords'
+                        help=argparse.SUPPRESS)
     parser.add_argument('--ls_region_format_keywords', action='store_true',
-                        help='list available region format keywords'
-                        if not hide_help else argparse.SUPPRESS)
+                        # help='list available region format keywords'
+                        help=argparse.SUPPRESS)
     if hide_help:
         if parser.epilog is None:
             parser.epilog = ''
         else:
             parser.epilog += ' '
         parser.epilog += ' '.join([
-            "Accesses format options from 'damo args accesses_format' are",
-            "also supported. Do 'damo args accesses_format -h' for the",
-            "options.",
+            "More options for customized format are available.",
+            "Do 'damo help access_format_options -h' for those options.",
             ])
 
 def set_argparser(parser):
@@ -1487,7 +1617,7 @@ def set_argparser(parser):
     _damon_args.set_common_argparser(parser)
 
     # what to show
-    _damo_records.set_filter_argparser(parser)
+    _damo_records.set_filter_argparser(parser, hide_help=True)
 
     parser.add_argument('--input_file', metavar='<file>',
             help='source of the access pattern to show')
@@ -1499,10 +1629,13 @@ def set_argparser(parser):
     add_fmt_args(parser, hide_help=True)
     parser.add_argument('--format', metavar='<json string>',
                         help='visualization format in json format')
+    parser.add_argument('--exec', metavar='<command or \'interpreter\'>',
+                        help='execute python code with the records')
     parser.add_argument(
             '--on_cache', nargs=3,
             metavar=('<cache size>', '<cache ways>', '<cache line size>'),
-            help='visualize access patterns on a virtual cache (EXPERIMENTAL)')
+            # visualize access patterns on a virtual cache (EXPERIMENTAL)
+            help=argparse.SUPPRESS)
     parser.add_argument(
             '--repeat', nargs='*', metavar=('<delay>', '<count>'),
             help='repeat <count> times with <delay> time interval')
